#############################################
# File Name: Signals.py
# Author: W-Mai
# Mail: 1341398182@qq.com
# Created Time:  2022-04-16
#############################################

# {
#   "label":      "CLK",
#   "descriptor": [0, 0],
#   "alias":      "CLK"
# }
from typing import Tuple, Union


class Descriptor(object):
    def __init__(self, start, end):
        self.start = start
        self.end = end

    def __str__(self):
        return f"{self.start}, {self.end}]"

    @property
    def export(self):
        return [self.start, self.end]


class SignalBase(object):
    TYPE = None

    def __init__(self, label: str, descriptor: Union[Descriptor, Tuple] = Descriptor(0, 0), alias: str = None):
        self._model = None
        self.label = label
        self.descriptor = Descriptor(*descriptor) if not isinstance(descriptor, Descriptor) else descriptor
        self.alias = alias

    @property
    def model(self):
        return self._model

    @model.setter
    def model(self, value):
        self._model = value

    def to_dict(self):
        return {
            "label": self.label,
            "descriptor": self.descriptor.export,
            "alias": self.alias
        }


class Wire(SignalBase):
    TYPE = "Wire"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class Reg(SignalBase):
    TYPE = "Reg"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)



#############################################
# File Name: FrameworkNode.py
# Author: W-Mai
# Mail: 1341398182@qq.com
# Created Time:  2022-04-16
#############################################
from optparse import Option
from typing import Dict, Type, Optional, List, Tuple
from .Signals import *


# {
#       "name":    "ModelName",
#       "signals": [],
# }
#

class Point(object):
    def __init__(self, x: float, y: float):
        self.x = x
        self.y = y

    def __str__(self):
        return "Point(x={}, y={})".format(self.x, self.y)

    @property
    def export(self):
        return {
            "x": self.x,
            "y": self.y
        }


class ModelBoxBaseModelMetaclass(type):
    def __new__(mcs, name, bases, attrs):
        mcs._attrs = None
        mcs._signals = None

        attrs["_attrs"] = attrs

        signals = []
        for attr in attrs.items():
            sig_name: str = attr[0]
            sig_obj = attr[1]
            if isinstance(sig_obj, SignalBase):
                # sig_obj: SignalBase
                if sig_obj.alias is None:
                    setattr(sig_obj, 'alias', sig_name)
                sig_obj.model = ""
                signals.append(sig_obj)
        attrs['_signals'] = signals

        obj = super().__new__(mcs, name, bases, attrs)
        for sig in obj._signals:
            sig.model = obj

        return obj


class ModelBoxBaseModel(object, metaclass=ModelBoxBaseModelMetaclass):
    class Meta:
        pass

    def __init__(self):
        pass

    @classmethod
    def to_dict(cls):
        meta_dict = cls.meta_dict()
        res_dict = {
            # 如果name为空，则使用类名
            'name': cls.name(),
            'signals': [sig.to_dict() for sig in cls.signals()]
        }

        return res_dict

    @classmethod
    def other_conf(cls):
        meta_dict = cls.meta_dict()
        res_dict = {
            conf_name:
                meta_dict.get(conf_name, None)
            for conf_name in cls.readable_conf() if meta_dict.get(conf_name, None) is not None
        }
        return res_dict

    # 哪些配置项可以读取
    @staticmethod
    def readable_conf():
        return ["position", "flag"]

    @classmethod
    def meta_dict(cls):
        meta_class = cls.Meta
        if meta_class is not None:
            return {
                conf_name: getattr(meta_class, conf_name, None)
                for conf_name in cls.readable_conf()
            }

    @classmethod
    def attrs(cls):
        return cls._attrs

    @classmethod
    def signals(cls):
        return cls._signals

    @classmethod
    def name(cls):
        return getattr(cls.Meta, "name", cls.raw_name())

    @classmethod
    def raw_name(cls):
        return cls.__name__


class Connector(object):
    def __init__(self, start_signal: SignalBase, end_signal: SignalBase, *positions: Tuple[int, int]):
        self.start_signal = start_signal
        self.end_signal = end_signal

        self.positions = []
        for pos in positions:
            self.positions.append(Point(*pos))

    def __str__(self):
        return "Connector(start={}, end={}, positions={})".format(self.start_signal, self.end_signal, self.positions)

    def to_dict(self):
        return {
            "from": f"{self.start_signal.model.name()}.{self.start_signal.label}",
            "to": f"{self.end_signal.model.name()}.{self.end_signal.label}",
            "positions": [pos.export for pos in self.positions]
        }


class CONFIGURE(object):

    def __init__(self,
                 node_pos_pair: Dict[Type[ModelBoxBaseModel], Tuple[int, int]],
                 connector_pair=None,
                 other_conf=None,
                 colors=None):
        if connector_pair is None:
            connector_pair = []
        if colors is None:
            colors = [
                "#ffa502", "#ff6348", "#ff4757", "#747d8c",
                "#2f3542", "#2ed573", "#1e90ff", "#3742fa",
                "#e84393", "#05c46b", "#ffd43b", "#ffa000"
            ]
        if other_conf is None:
            other_conf = dict()
        self.POSITIONS_PAIR = node_pos_pair
        self.COLORS = colors
        self.OTHER_CONF = other_conf
        self.CONNECTOR_PAIR = connector_pair

        for node, pos in node_pos_pair.items():
            node.Meta.position = Point(*pos)

        for node, conf in other_conf.items():
            for attr in conf.items():
                setattr(node.Meta, attr[0], attr[1])



from FrameworkDrawer.FrameworkNode import ModelBoxBaseModel, Reg, Wire, CONFIGURE, Connector


class ALU(ModelBoxBaseModel):
    CLK = Wire('CLK')
    RST = Wire('RST')
    D = Wire('D')
    Q = Wire('Q')
    QN = Wire('QN')


class Core_Ctrl(ModelBoxBaseModel):
    CLK = Wire('CLK')
    Jump_Flag_In = Wire('Jump_Flag_In66')
    Jump_Addr_In = Wire('Jump_Addr_In66', (0, 31))

    Hold_Flag_EX_In = Wire('Hold_Flag_EX_In')
    Hold_Flag_RIB_In = Wire('Hold_Flag_RIB_In')
    Hold_Flag_CLINT_In = Wire('Hold_Flag_CLINT_In')

    Jtag_Halt_Flag_In = Wire('Jtag_Halt_Flag_In')

    Hold_Flag_Out = Wire('Hold_Flag_Out', (0, 2))
    Jump_Flag_Out = Wire('Jump_Flag_Out')
    Jump_Addr_Out = Wire('Jump_Addr_Out', (0, 31))

    class Meta:
        name = 'Core/Ctrl'


class Core_PC_REG(ModelBoxBaseModel):
    CLK = Wire('CLK')
    RST = Wire('RST')

    Jump_Flag_In = Wire('Jump_Flag_In')
    Jump_Addr_In = Wire('Jump_Addr_In', (0, 31))
    Hold_Flag_In = Wire('Hold_Flag_In', (0, 2))
    Jtag_Reset_Flag_In = Wire('Jtag_Reset_Flag_In')

    PC_Out = Wire('PC_Out', (0, 31))

    class Meta:
        name = 'Core/PC_REG'


class Core_REGS(ModelBoxBaseModel):
    CLK = Wire('CLK')
    RST = Wire('RST')

    Write_En_In = Wire('Write_En_In')
    Write_Addr_In = Wire('Write_Addr_In', (0, 4))
    Write_Data_In = Wire('Write_Data_In', (0, 31))

    Read_Addr1_In = Wire('Read_Addr1_In', (0, 4))
    Read_Addr2_In = Wire('Read_Addr2_In', (0, 4))

    Read_Data1_Out = Wire('Read_Data1_Out', (0, 31))
    Read_Data2_Out = Wire('Read_Data2_Out', (0, 31))

    class Meta:
        name = 'Core/REGS'


class Core_CSR_REG(ModelBoxBaseModel):
    CLK = Wire('CLK')
    RST = Wire('RST')

    Write_En_In = Wire('Write_En_In')
    Write_Addr_In = Wire('Write_Addr_In', (0, 31))
    Write_Data_In = Wire('Write_Data_In', (0, 31))

    Read_Addr_In = Wire('Read_Addr_In', (0, 31))

    Data_Out = Wire('Data_Out', (0, 31))

    CLINT_Write_En_In = Wire('CLINT_Write_En_In')
    CLINT_Write_Addr_In = Wire('CLINT_Write_Addr_In', (0, 31))
    CLINT_Write_Data_In = Wire('CLINT_Write_Data_In', (0, 31))
    CLINT_Read_Addr_In = Wire('CLINT_Read_Addr_In', (0, 31))
    CLINT_Data_Out = Wire('CLINT_Data_Out', (0, 31))
    CLINT_CSR_MTVEC = Reg('CLINT_CSR_MTVEC', (0, 31))
    CLINT_CSR_MEPC = Reg('CLINT_CSR_MEPC', (0, 31))
    CLINT_CSR_MSSTATUS = Reg('CLINT_CSR_MSSTATUS', (0, 31))

    class Meta:
        name = 'Core/CSR_REG'


class Core_IF_ID(ModelBoxBaseModel):
    CLK = Wire('CLK')
    RST = Wire('RST')

    Instruction_In = Wire('Instruction_In', (0, 31))
    Instruction_Addr_In = Wire('Instruction_Addr_In', (0, 31))

    Hold_Flag_In = Wire('Hold_Flag_In')
    Int_Flag_In = Wire('Int_Flag_In')

    Int_Flag_Out = Wire('Int_Flag_Out')
    Instruction_Out = Wire('Instruction_Out', (0, 31))
    Instruction_Addr_Out = Wire('Instruction_Addr_Out', (0, 31))

    class Meta:
        name = 'Core/IF_ID'


class Core_ID(ModelBoxBaseModel):
    CLK = Wire('CLK')

    Instruction_In = Wire('Instruction_In', (0, 31))
    Instruction_Addr_In = Wire('Instruction_Addr_In', (0, 31))

    Reg1_Read_Data_In = Wire('Reg1_Read_Data_In', (0, 31))
    Reg2_Read_Data_In = Wire('Reg2_Read_Data_In', (0, 31))

    CSR_Read_Data_In = Wire('CSR_Read_Data_In', (0, 31))

    EX_Jump_Flag_In = Wire('EX_Jump_Flag_In')
    Reg1_Read_Addr_Out = Wire('Reg1_Read_Addr_Out', (0, 4))
    Reg2_Read_Addr_Out = Wire('Reg2_Read_Addr_Out', (0, 4))

    CSR_Read_Addr_Out = Wire('CSR_Read_Addr_Out', (0, 31))
    MEM_Req_Out = Wire('MEM_Req_Out')

    Instruction_Out = Wire('Instruction_Out', (0, 31))
    Instruction_Addr_Out = Wire('Instruction_Addr_Out', (0, 31))
    Reg1_Read_Data_Out = Wire('Reg1_Read_Data_Out', (0, 31))
    Reg2_Read_Data_Out = Wire('Reg2_Read_Data_Out', (0, 31))

    Reg_Write_En_Out = Wire('Reg_Write_En_Out')
    Reg_Write_Addr_Out = Wire('Reg_Write_Addr_Out', (0, 4))
    CSR_Write_En_Out = Wire('CSR_Write_En_Out')
    CSR_Read_Data_Out = Wire('CSR_Read_Data_Out', (0, 31))
    CSR_Write_Addr_Out = Wire('CSR_Write_Addr_Out', (0, 31))

    class Meta:
        name = 'Core/ID'


CONFIG = CONFIGURE(
    node_pos_pair={
        ALU: (-20, 10),
        Core_Ctrl: (-6.5, 5),
        Core_PC_REG: (-35, 0),
        Core_REGS: (-45, 15),
        Core_CSR_REG: (-55, 15),
        Core_IF_ID: (-26, 5),
        Core_ID: (-15, 2),
    }, other_conf={
        ALU: {
            'flag': True
        },
        Core_IF_ID: {
            'flag': True
        },
    }, connector_pair=[
        Connector(Core_Ctrl.CLK, Core_PC_REG.CLK, (-4, 15)),
        Connector(Core_Ctrl.Jump_Addr_In, Core_PC_REG.Jump_Addr_In, (-4, 16)),
    ], colors=[
        "#ffa502", "#ff6348", "#ff4757", "#747d8c",
        "#2f3542", "#2ed573", "#1e90ff", "#3742fa",
        "#e84393", "#05c46b", "#ffd43b", "#ffa000"
    ]
)



#############################################
# File Name: FrameworkDrawer.py
# Author: W-Mai
# Mail: 1341398182@qq.com
# Created Time:  2022-04-11
#############################################
import inspect
import io
import json
from typing import IO, Union, Literal

import schemdraw
import schemdraw.elements as elm
from schemdraw import Segment, SegmentCircle, SegmentText, ImageFormat
from schemdraw.util import Point
from itertools import combinations, chain, tee
from xunionfind import UnionFind

from FrameworkDrawer.FrameworkNode import CONFIGURE, ModelBoxBaseModel
from FrameworkDrawer.FrameworkNode import Point as FPoint
from FrameworkDrawer.Signals import SignalBase


def pairwise(iterable):
    """s -> (s0,s1), (s1,s2), (s2, s3), ..."""
    a, b = tee(iterable)
    next(b, None)
    return zip(a, b)


class ConnectorBox(elm.Element):
    def __init__(self, source: str, target: str, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.source = source
        self.target = target
        self.label = f"{source}->{target}"

        self.anchors['center'] = (0, 0)


class ModelBox(elm.Element):
    def __init__(self, name, signals=None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if signals is None:
            signals = []

        self.model_name = SegmentText((0, 0.5), name, fontsize=21)
        self.segments.append(self.model_name)

        real_width = (self.model_name.get_bbox().xmin + self.model_name.get_bbox().xmax) / 2

        for signal in signals:
            tmp_descriptor = signal['descriptor']
            descriptor = "" if tmp_descriptor[0] == 0 and tmp_descriptor[
                1] == 0 else f" [{tmp_descriptor[1]}, {tmp_descriptor[0]}]"

            sig_text = SegmentText((0, -0.6 * (signals.index(signal)) - 0.3), f"{signal['alias']}{descriptor}",
                                   fontsize=12)
            self.segments.append(sig_text)

            delta_width = (sig_text.get_bbox().xmin + sig_text.get_bbox().xmax) / 2
            if delta_width > real_width:
                real_width = delta_width

        padding_width = real_width + 0.5

        self.segments.append(Segment([(-padding_width, 1), (padding_width, 1)]))
        self.segments.append(Segment([(-padding_width, 0), (padding_width, 0)]))

        delta_y = 0
        for index in range(0, len(signals)):
            signal = signals[index]
            delta_y = -0.6 * (signals.index(signal)) - 0.6

            self.segments.append(SegmentCircle((-padding_width, delta_y + 0.3), 0.06, fill=True))
            self.segments.append(SegmentCircle((padding_width, delta_y + 0.3), 0.06, fill=True))
            self.anchors[signal['label'] + ".left"] = (-padding_width, delta_y + 0.3)
            self.anchors[signal['label'] + ".right"] = (padding_width, delta_y + 0.3)

            if index < len(signals) - 1:
                self.segments.append(Segment([(-padding_width, delta_y), (padding_width, delta_y)], lw=1))

        self.segments.append(Segment([(-padding_width, 1), (-padding_width, delta_y)]))
        self.segments.append(Segment([(padding_width, 1), (padding_width, delta_y)]))
        self.segments.append(Segment([(-padding_width, delta_y), (padding_width, delta_y)]))


class FrameworkDrawer(object):
    def __init__(self, file, is_model=False):
        self.file = file
        self.output_file = None
        if not is_model:
            if file is None:
                raise ValueError("json_file is Not Found")
            if isinstance(file, str):
                self.json_data = json.loads(file)
            else:
                self.json_data = json.load(file)
        else:
            if file is None:
                raise ValueError("Model is Not Found")
            buffer = io.StringIO()
            self.export_model_to_json(buffer)
            self.json_data = json.loads(buffer.getvalue())

    def draw(self, output_file, fmt: Union[
        Literal['eps', 'jpg', 'pdf', 'pgf', 'png', 'ps', 'raw', 'rgba', 'svg', 'tif'], ImageFormat] = "svg"):
        if output_file is None:
            raise ValueError("output_file is None")

        with schemdraw.Drawing(show=False) as d:
            model_json = self.json_data

            COLORS = model_json['colors']
            connectors = model_json['connectors']
            connector_instance = dict()

            for connector in connectors:
                tmp_connectors = []
                for connector_pos in connector['positions']:
                    tmp_connectors.append(
                        d.add(ConnectorBox(connector['from'], connector['to']).at(connector_pos.values())))
                if len(tmp_connectors) == 0:
                    continue
                connector_instance[tmp_connectors[0].label] = tmp_connectors

            model_info = []
            model_instances = dict()

            for model in model_json['models']:
                pos = Point(model_json['drawing_info'][model['name']]['position'].values())
                flag = model_json['drawing_info'][model['name']].get('flag', False)
                model_instances[model['name']] = d.add(ModelBox(model['name'], model['signals']).at(pos))

                model_info.append({
                    'name': model['name'],
                    'pos': pos,
                    'flag': flag,
                    'signals': [s['label'] for s in model['signals']],
                })

            all_signals = {s for info in model_info for s in info['signals']}
            all_signals = chain(all_signals, [c for c in connector_instance])
            signal_color_map = {s: COLORS[i % len(COLORS)] for i, s in enumerate(sorted(all_signals))}
            # 1,2,3,4 combinations is
            # [1, 2] [1, 3] [1, 4] [2, 3] [2, 4] [3, 4]
            # [1, 2] [2, 3] [3, 4] [1, 3] [2, 4] [1, 4]
            model_info.sort(key=lambda x: x['pos'].x)
            model_combination = combinations(model_info, 2)
            model_combination = list(model_combination)

            model_wire_signal_model_set = dict()
            next_model_name = model_info[0]['name']
            while len(model_combination) > 0:
                try:
                    model_pair = next(filter(lambda m: m[0]['name'] == next_model_name, model_combination))
                except StopIteration:
                    model_pair = model_combination[0]
                next_model_name = model_pair[1]['name']
                model_combination.remove(model_pair)

                model_pair = list(model_pair)
                model_pair.sort(key=lambda x: x['pos'].x)

                left_model = model_pair[0]
                right_model = model_pair[1]

                def get_signal(model_name, signal_name, direction):
                    if direction == "left":
                        return getattr(model_instances[model_name], f"{signal_name}.left")
                    else:
                        return getattr(model_instances[model_name], f"{signal_name}.right")

                for signal in set(left_model['signals']).intersection(right_model['signals']):
                    if signal not in model_wire_signal_model_set:
                        model_wire_signal_model_set[signal] = UnionFind()
                    if model_wire_signal_model_set[signal].connected(left_model['name'], right_model['name']):
                        continue

                    left_signal_index = left_model['signals'].index(signal)
                    right_signal_index = right_model['signals'].index(signal)

                    model_wire_signal_model_set[signal].union(left_model['name'], right_model['name'])
                    left_model_left_signal = get_signal(left_model['name'], signal, 'left')
                    left_model_right_signal = get_signal(left_model['name'], signal, 'right')
                    right_model_left_signal = get_signal(right_model['name'], signal, 'left')
                    right_model_right_signal = get_signal(right_model['name'], signal, 'right')

                    if left_model_right_signal.x + 0.5 > right_model_left_signal.x:
                        # 当左模块的右边信号跟右模块的左边信号重合时
                        if right_model['flag']:
                            wire = elm.Wire(shape="c",
                                            k=right_model_right_signal.x - left_model_right_signal.x + 0.5 + left_signal_index / 4)
                            d.add(wire
                                  .at(left_model_right_signal)
                                  .to(right_model_right_signal)
                                  .color(signal_color_map[signal]))
                        else:
                            wire = elm.Wire(shape="c",
                                            k=-(0.5 + left_signal_index / 4))
                            d.add(wire
                                  .at(left_model_left_signal)
                                  .to(right_model_left_signal)
                                  .color(signal_color_map[signal]))
                    else:
                        # 当左模块的右边信号跟右模块的左边信号不重合时
                        k_left = 0.5 + left_signal_index / 4
                        k_right = 0.5 + right_signal_index / 4

                        if left_model_right_signal.y < right_model_left_signal.y:
                            if k_left < right_model_left_signal.x - left_model_right_signal.x:
                                d.add(elm.Wire("c", k=k_left).at(left_model_right_signal).to(
                                    right_model_left_signal)).color(signal_color_map[signal])
                            else:
                                d.add(elm.Wire("c", k=k_right).at(left_model_right_signal).to(
                                    right_model_left_signal)).color(signal_color_map[signal])

                        else:
                            if k_left < right_model_left_signal.x - left_model_right_signal.x:
                                d.add(elm.Wire("c", k=-k_left).at(right_model_left_signal).to(
                                    left_model_right_signal)).color(signal_color_map[signal])
                            else:
                                d.add(elm.Wire("c", k=-k_right).at(right_model_left_signal).to(
                                    left_model_right_signal)).color(signal_color_map[signal])

        for label, connectors in connector_instance.items():
            for con1, con2 in pairwise(connectors):
                k0 = (con1.center.x - con2.center.x) / 2
                d.add(elm.Wire("c", k=k0).at(con2.center).to(con1.center).color(signal_color_map[label]))
                # print(con1, con2, con1.center, con2.center)

            # 解析模块和信号

            def get_model_info(model_name):
                for model_i in model_info:
                    if model_i["name"] == model_name:
                        return model_i
                return None

            label: str
            (
                start_model_name, start_model_signal_name), (
                end_model_name, end_model_signal_name
            ) = [label.split('.') for label in label.split('->')]

            start_model_info = get_model_info(start_model_name)
            end_model_info = get_model_info(end_model_name)

            if start_model_info is None or end_model_info is None:
                continue

            start_signal_left = get_signal(start_model_name, start_model_signal_name, 'left')
            start_signal_right = get_signal(start_model_name, start_model_signal_name, 'right')
            end_signal_left = get_signal(end_model_name, end_model_signal_name, 'left')
            end_signal_right = get_signal(end_model_name, end_model_signal_name, 'right')

            start_connector = connectors[0]
            end_connector = connectors[-1]

            start_signal_index = start_model_info["signals"].index(start_model_signal_name)
            end_signal_index = end_model_info["signals"].index(end_model_signal_name)

            k0 = 0.5 + start_signal_index / 4
            if start_signal_left.x < start_connector.center.x < start_signal_right.x:
                d.add(elm.Wire("c", k=k0).at(start_signal_right).to(start_connector.center).color(
                    signal_color_map[label]))
            else:
                if start_signal_left.x > start_connector.center.x:
                    d.add(elm.Wire("c", k=-k0).at(start_signal_left).to(start_connector.center).color(
                        signal_color_map[label]))
                else:
                    d.add(elm.Wire("c", k=k0).at(start_signal_right).to(start_connector.center).color(
                        signal_color_map[label]))

            k0 = 0.5 + end_signal_index / 4
            if end_signal_left.x < end_connector.center.x < end_signal_right.x:
                d.add(elm.Wire("c", k=k0).at(end_signal_left).to(end_connector.center).color(signal_color_map[label]))
            else:
                if end_signal_left.x > end_connector.center.x:
                    d.add(elm.Wire("c", k=-k0).at(end_signal_left).to(end_connector.center).color(
                        signal_color_map[label]))
                else:
                    d.add(elm.Wire("c", k=k0).at(end_signal_right).to(end_connector.center).color(
                        signal_color_map[label]))

        if isinstance(output_file, str):
            d.save(output_file)
        if isinstance(output_file, IO):
            output_file.write(d.get_imagedata(fmt=fmt))

    def export_model_to_json(self, output_file, indent=None):
        module = self.file
        configures = inspect.getmembers(module, lambda member: isinstance(member, CONFIGURE))
        if len(configures) == 0:
            raise Exception('CONFIGURE class not found.')
        configure: CONFIGURE = configures[0][1]

        framework_draw_file_dict = {
            'drawing_info': {},
            'models': [],
            'connectors': [connector.to_dict() for connector in configure.CONNECTOR_PAIR],
            'colors': configure.COLORS,
        }

        for name, model in inspect.getmembers(module, lambda member: inspect.isclass(member)
                                                                     and issubclass(member, ModelBoxBaseModel)
                                                                     and not member == ModelBoxBaseModel):
            model_ins = model
            framework_draw_file_dict['models'].append(model_ins.to_dict())
            framework_draw_file_dict['drawing_info'][model_ins.name()] = model_ins.other_conf()

        if isinstance(output_file, str):
            json.dump(framework_draw_file_dict, open(output_file, 'w', encoding="utf-8"), default=lambda o: o.export,
                      indent=indent)
        else:
            json.dump(framework_draw_file_dict, output_file, default=lambda o: o.export, indent=indent)
        # print(framework_draw_file_dict)



#############################################
# File Name: FrameworkDrawer.py
# Author: W-Mai
# Mail: 1341398182@qq.com
# Created Time:  2022-04-13
#############################################
import time

from FrameworkDrawer import FrameworkDrawer
from FrameworkDrawer.FrameworkNode import Connector

if __name__ == '__main__':
    t0 = time.time()
    fd = FrameworkDrawer(file=__import__("MyNode"), is_model=True)
    fd.draw("../../imgs/schema.svg")
    print("Time:", time.time() - t0)

    #
    # fd = FrameworkDrawer(file=MyNode, is_model=True)
    # fd.export_model_to_json(open("../../model.json", "w"))



# -*- coding: utf-8 -*-
#############################################
# File Name: setup.py
# Author: W-Mai
# Mail: 1341398182@qq.com
# Created Time:  2022-04-11
#############################################
from setuptools import setup, find_packages

long_description = """# FrameworkDrawer
一个绘制简单RTL门级电路的模块

使用方法如下：
```bash
python main.py [-h] --input INPUT --output OUTPUT [--fmt FMT]
```

用例：
```bash
 python .\main.py --input .\model.json --output .\imgs\schema.svg
```

输出的图片效果如下
<div style="background-color: white;">
<img src="https://raw.githubusercontent.com/W-Mai/FrameworkDrawer/63f0cfa34c3bb4190a72188e002fc8b9b2347516/imgs/schema.svg">
</div>
"""

setup(
    name="FrameworkDrawer",
    version="0.0.6",
    description="Draw a simple RTL block diagram.",
    long_description_content_type="text/markdown",
    long_description=long_description,
    author="W-Mai",
    author_email="1341398182@qq.com",
    url="https://github.com/W-Mai/FrameworkDrawer",
    license="MIT Licence",
    packages=find_packages(),
    install_requires=['schemdraw[svgmath]', 'pillow', 'xunionfind'],
)
